@@ -704,6 +704,14 @@ public:
 	 */
 	static void allocateLocal(void);
 
+	/**
+	 * Compute md5 hashes...
+	 *
+	 * @param md5 output string
+	 * @param string to hash
+	 */
+	void md5_hash(char *out, const char *source);
+
 	/**
 	 * Wait for live flag...
 	 */
