@@ -27,7 +27,7 @@ BayonneDriver *BayonneDriver::protoDriver = NULL;
 Semaphore BayonneDriver::oink(0);
 
 BayonneDriver::BayonneDriver(Keydata::Define *pairs, const char *cfg, const char *id, bool virt) :
-Keydata(pairs, getKeypath(cfg)), Mutex()
+Keydata(pairs, cfg), Mutex()
 {
 	name = id;
 	running = false;
@@ -251,28 +251,9 @@ BayonneDriver *BayonneDriver::loadDriver(const char *id)
 	const char *prefix = NULL;
 
 #ifdef	WIN32
-#ifdef	HAVE_TESTING
-	if(!prefix && provision_test)
-#ifdef	_DEBUG
-		prefix = "DEBUG";
-#else
-		prefix = "RELEASE";
-#endif
-#endif
 	if(!prefix)
 		prefix="C:\\Program Files\\Common Files\\GNU Telephony\\Bayonne Drivers";
 #else
-
-#ifdef	HAVE_TESTING
-	char prefixbuf[256];
-	if(!prefix && provision_test)
-	{
-		snprintf(prefixbuf, sizeof(prefixbuf), "%s/modules/%s/.libs", SOURCE_FILES, id);
-		if(!isDir(prefixbuf))
-			snprintf(prefixbuf, sizeof(prefixbuf), "%s/modules/%s", SOURCE_FILES, id);
-		prefix = prefixbuf;
-	}
-#endif
 	if(!prefix)
 		prefix=LIBDIR_FILES;
 #endif
