@@ -97,12 +97,15 @@ BayonneSession::~BayonneSession()
 	finalize();
 }
 
-Script::Name *BayonneSession::getScript(const char *scr)
+/* Script::Name *BayonneSession::getScript(const char *scr)
 {
 	ScriptImage **img;
 
+	printf("GET SCRIPT!!!! %s\n", scr);
+
 	if(!strnicmp(scr, "_local_::", 9))
 	{
+		printf("LOCAL!\n");
 		img = getLocalImage(timeslot);
 		if(!img)
 			return NULL;
@@ -115,6 +118,7 @@ Script::Name *BayonneSession::getScript(const char *scr)
 
 	return ScriptInterp::getScript(scr);
 } 
+*/
 
 void BayonneSession::initialize(void)
 {
