@@ -29,20 +29,8 @@ Keydata()
 {
 	char name[128];
 
-#ifdef	WIN32
-	snprintf(name, sizeof(name), "/bayonne/span%d", spans);
-	load(name);
-        snprintf(name, sizeof(name), "%d", spans);
-	const char *env = Process::getEnv("SPANCONFIG");
-
-	if(env)
-		loadFile(env, name);
-
-#else
 	snprintf(name, sizeof(name), "/bayonne/spans/%d", spans);
-	load(getKeypath(name));
-
-#endif
+	load(name);
 
 	id = spans++;
 	if(first)
