@@ -38,31 +38,12 @@ static Keydata::Define driver[] = {
 	{"mode", "cpe"},
 	{NULL, NULL}};
 
-#ifdef  WIN32
-#define KEYS    "/bayonne/sangoma"
-#else
-#define KEYS    "/bayonne/driver/sangoma"
-#endif
-
 Driver Driver::sangoma;
 
 Driver::Driver() :
-BayonneDriver(driver, KEYS, "sangoma", false)
+BayonneDriver(driver, "/bayonne/driver/sangoma", "sangoma", false)
 {
         const char *cp;
-
-#ifdef  WIN32
-        const char *env = Process::getEnv("DRIVERCONFIG");
-
-        if(env)
-                loadFile(env, "sangoma");
-#else
-        if(Bayonne::provision_system)
-                load("/bayonne/provision/sangoma");
-        else if(Bayonne::provision_user)
-                load("~bayonne/sangoma");
-#endif
-
 	spancount = 0;
 
 	pri_switch = SANGOMA_PRI_SWITCH_UNKNOWN;
