@@ -16,7 +16,6 @@
 
 #include "driver.h"
 
-
 #ifdef  WIN32
 #define CONFIG_FILES    "C:/Program Files/GNU Telephony/Bayonne Config"
 #define	strdup	ost::strdup
@@ -46,7 +45,7 @@ static Keydata::Define driver[] = {
 	{"invite", "60000"},
 	{"pickup", "2000"},
 	{"hangup", "250"},
-	{"accept", "320"},
+	{"accept", "600"},
 	{"audio", "60"},
 	{"jitter", "2"},
 	{"filler", "false"},
@@ -54,17 +53,35 @@ static Keydata::Define driver[] = {
 	{"immediate", "yes"},
 	{"peering", "friend"},	// default registration type
 	{"starting", "delayed"},
+	{"realm", "Bayonne"},
+	{"nonce", "456345766asd345"},
 #ifdef	WIN32		// use localhost default if not in conf for w32
 	{"interface", "127.0.0.1:5070"},
 #endif
         {NULL, NULL}};
 
+static char *remove_quotes(char *c)
+{
+	char *o = c;
+	char *d = c;
+	if(*c != '\"')
+		return c;
+
+	++c;
+
+	while(*c)
+		*(d++) = *(c++); 
+
+	*(--d) = 0;
+	return o;
+}
+
 Driver Driver::sip;
 
 bool Registry::isActive(void)
 {
 	if(!proxy)
-		return active;
+		return false;
 
 	if(!active)
 		return false;
@@ -84,7 +101,7 @@ void Registry::add(ScriptImage *img)
 
 Driver::Driver() :
 BayonneDriver(driver, "/bayonne/driver/sip", "sip", false),
-Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024)
+Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024 + 8192)
 {
 	const char *cp, *p;
 	char buf[1024];
@@ -579,6 +596,191 @@ const char *Driver::assignScript(ScriptImage *img, Line *line)
 	return "";
 }
 
+void Driver::requestAuth(eXosip_event_t *sevent)
+{
+	osip_message_t *reply = NULL;
+        const char *realm = getLast("realm");
+        const char *nonce = getLast("nonce");
+	char buf[256];
+
+	eXosip_lock();
+	eXosip_call_build_answer(sevent->tid, 407, &reply);
+        snprintf(buf, sizeof(buf),
+                "Digest realm=\"%s\", nonce=\"%s\"", realm, nonce);
+        osip_message_set_header(reply, "Proxy-Authenticate", buf);
+        eXosip_call_send_answer(sevent->tid, 407, reply);
+        eXosip_unlock();
+}
+
+bool Driver::getRegistry(eXosip_event_t *sevent, Registry **reg, ScriptImage *img)
+{
+	char a1[256], a2[256], a1_hash[256], a2_hash[256], resp[256], resp_hash[256];
+	osip_proxy_authorization_t *auth;
+	char buf[256];
+
+	*reg = NULL;	
+
+	eXosip_lock();
+        if(osip_message_get_proxy_authorization(sevent->request, 0, &auth) != 0)
+	{
+		eXosip_unlock();
+		return true;
+	}
+
+	eXosip_unlock();
+
+	if(!auth)
+		return true;
+
+	if(!auth->username)
+		return false;
+
+        remove_quotes(auth->username);
+        remove_quotes(auth->uri);
+        remove_quotes(auth->nonce);
+        remove_quotes(auth->response);
+        snprintf(buf, sizeof(buf), "uri.%s", auth->username);
+	*reg = (Registry *)img->getPointer(buf);
+
+	if(!*reg)
+		return false;
+
+	snprintf(a1, sizeof(a1), "%s:%s:%s",
+                auth->username, getLast("realm"), (*reg)->secret);
+        snprintf(a2, sizeof(a2), "INVITE:%s", auth->uri);
+        md5_hash(a1_hash, a1);
+        md5_hash(a2_hash, a2);
+        snprintf(resp, sizeof(resp), "%s:%s:%s",
+                a1_hash, auth->nonce, a2_hash);
+        md5_hash(resp_hash, resp);
+	if(!stricmp(auth->response, resp_hash))
+		return true;
+	return false;
+}
+
+void Driver::registerRequest(eXosip_event_t *sevent)
+{
+	char a1[256], a2[256], a1_hash[256], a2_hash[256], resp[256], resp_hash[256];
+	osip_message_t *reply = NULL;
+	osip_contact_t *contact = NULL;
+	osip_authorization_t *auth = NULL;
+	osip_header_t *exp = NULL;
+	const char *realm = getLast("realm");
+	const char *nonce = getLast("nonce");
+	char buf[256];
+	ScriptImage *img;
+	Registry *reg;	
+	int pos = 0;
+	char *cp;
+
+	eXosip_lock();
+	if(osip_message_get_authorization(sevent->request, 0, &auth) != 0)
+		goto challenge;
+
+	if(!auth || !auth->username || !auth->response)
+		goto challenge;
+
+	eXosip_unlock();
+	remove_quotes(auth->username);
+	remove_quotes(auth->uri);
+	remove_quotes(auth->nonce);
+	remove_quotes(auth->response);
+	snprintf(buf, sizeof(buf), "uri.%s", auth->username);
+
+	server->enter();
+	img = server->getActive();
+	reg = (Registry *)img->getPointer(buf);	
+	if(!reg || strnicmp(reg->type, "ext", 3))
+	{
+fail401:
+
+		server->leave();
+		slog.debug("rejecting external %s", auth->username); 
+		eXosip_lock();
+		eXosip_message_build_answer(sevent->tid, 401, &reply);
+		eXosip_message_send_answer(sevent->tid, 401, reply);
+		eXosip_unlock();		
+		return;
+	}
+	snprintf(a1, sizeof(a1), "%s:%s:%s",
+		auth->username, getLast("realm"), reg->secret);
+	snprintf(a2, sizeof(a2), "REGISTER:%s", auth->uri);
+	md5_hash(a1_hash, a1);
+	md5_hash(a2_hash, a2);
+	snprintf(resp, sizeof(resp), "%s:%s:%s",
+		a1_hash, auth->nonce, a2_hash);
+	md5_hash(resp_hash, resp);
+	if(stricmp(auth->response, resp_hash))
+		goto fail401;
+
+	while(osip_list_eol(sevent->request->contacts, pos) == 0)
+	{
+		contact = (osip_contact_t *)osip_list_get(sevent->request->contacts, pos++);
+		if(contact && contact->url)
+			break;
+	}
+
+	if(!contact || !contact->url)
+		goto fail401;
+
+	if(!stricmp(reg->type, "peer"))
+		goto sip200;
+
+	snprintf(buf, sizeof(buf), 
+		"%s:%s", contact->url->host, contact->url->port);
+
+	if(!reg->proxy || stricmp(buf, reg->proxy))
+	{
+		cp = (char *)img->getMemory(strlen(buf));
+		strcpy(cp, buf);
+		reg->proxy = cp;
+	}
+
+	if(!reg->userid || stricmp(contact->url->username, reg->userid))
+	{
+		cp = (char *)img->getMemory(strlen(contact->url->username) + 1);
+		strcpy(cp, contact->url->username);
+		reg->userid = cp;
+	}
+
+sip200:
+	server->leave();
+	eXosip_lock();	
+	eXosip_message_build_answer(sevent->tid, 200, &reply);
+	eXosip_message_send_answer(sevent->tid, 200, reply);
+
+	osip_message_get_expires(sevent->request, 0, &exp);
+	if(exp && exp->hvalue)
+		pos = osip_atoi(exp->hvalue);
+	else
+		pos = 600;
+
+	if(pos < 0)
+		pos = 600;
+	eXosip_unlock();
+	if(pos)
+	{
+		reg->setTimer(pos * 1000);
+		reg->active = true;
+		slog.debug("registered external %s from %s; expires=%d", auth->username, buf, pos);
+	}
+	else
+	{
+		reg->active = false;
+		reg->setTimer(0);
+		slog.debug("unregister external %s from %s", auth->username, buf);
+	}
+	return;
+
+challenge:
+	eXosip_message_build_answer(sevent->tid, 401, &reply);
+	snprintf(buf, sizeof(buf), 
+		"Digest realm=\"%s\", nonce=\"%s\"", realm, nonce);
+	osip_message_set_header(reply, "WWW-Authenticate", buf);
+	eXosip_message_send_answer(sevent->tid, 401, reply);
+	eXosip_unlock();
+}
+
 const char *Driver::registerScript(ScriptImage *img, Line *line)
 {
 	Name *scr = img->getCurrent();
@@ -592,6 +794,15 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	char *uri;
 	long mult = 60000;
 	osip_message_t *msg = NULL; 
+	InetAddress host;
+	bool ext = false;
+
+	if(!strnicmp(line->cmd, "ext", 3))
+		ext = true;
+
+	if(ext)
+		if(!ScriptChecks::useKeywords(line, "=secret=encoding=framing=dtmf"))
+			return "invalid keywords used for external";
 
 	if(!ScriptChecks::useKeywords(line, "=uri=timeout=server=proxy=userid=secret=type=realm=dtmf=public=encoding=framing"))
 		return "invalid keywords for sip, use uri, timeout, proxy, userid, secret, port";
@@ -622,17 +833,23 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(!reg->dtmf)
 		reg->dtmf = "auto";
 
-	if(!reg->type)
-		reg->type = getLast("peering");
-
 	if(!reg->hostid || !*reg->hostid)
 		reg->hostid = ScriptChecks::findKeyword(line, "server");
 
 	if(!reg->hostid || !*reg->hostid)
-		reg->hostid = getLast("proxy");
+	{
+		if(!reg->type && ext)
+			reg->type = "extern";
+		else
+		{
+			reg->hostid = getLast("proxy");
+			if(!reg->hostid || !*reg->hostid)
+				reg->hostid = server->getLast("proxy");
+		}
+	}
 
-	if(!reg->hostid || !*reg->hostid)
-		reg->hostid = server->getLast("proxy");
+	if(!reg->type)
+		reg->type = getLast("peering");
 
 	if(!reg->hostid || !*reg->hostid)
 		reg->hostid = server->getLast("server");
@@ -743,6 +960,13 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(!reg->uri)
 		reg->uri = uri;
 
+	if(!stricmp(reg->type, "node") || !strnicmp(reg->type, "ext", 3))
+	{
+		reg->type = "extern";
+		slog.debug("register %s externally", reg->localid);
+		goto add;
+	}
+
 	slog.debug("register %s as %s on %s", reg->localid, reg->userid, reg->proxy);
 
 	if(!registry)
@@ -792,7 +1016,7 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	p = strchr(buf, ':');
 	if(p)
 		*p = 0;
-	InetAddress host(buf + 4);
+	host = (buf + 4);
 	if(p)
 		*p = ':';
 	else
@@ -804,6 +1028,8 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 
 	snprintf(buf, sizeof(buf), "sipreg.%d", reg->regid);
 	img->setPointer(buf, reg);
+
+add:
 	snprintf(buf, sizeof(buf), "uri.%s", reg->localid);
 	img->setPointer(buf, reg);
 	img->addRegistration(line);
@@ -1229,6 +1455,12 @@ killit:
 			if(event.id)
 				session->postEvent(&event);
 			break;
+		case EXOSIP_MESSAGE_NEW:
+			if(!sevent->request)
+				break;
+			if(MSG_IS_REGISTER(sevent->request))
+				registerRequest(sevent);
+			break;
 		case EXOSIP_CALL_INVITE:
 			if(!sevent->request)
 				break;
@@ -1308,10 +1540,34 @@ killit:
 			if(peer && !peer->isActive())
 				goto sip404;
 
-			snprintf(buf, sizeof(buf), "uri.%s", dialed);
-			reg = (Registry *)img->getPointer(buf);
+			reg = NULL;
+			if(!getRegistry(sevent, &reg, img))
+				goto sip401;
+
 			reg_inband = dtmf_inband;
 			reg_payload = dtmf_negotiate;
+
+			if(!reg)
+			{
+				snprintf(buf, sizeof(buf), "uri.%s", dialed);
+				reg = (Registry *)img->getPointer(buf);
+			}
+			else	
+			{
+				peer = reg;
+				if(!strnicmp(reg->type, "ext", 3))
+				{
+					auth = "peer";
+					goto validate;
+				}	
+				if(!stricmp(reg->type, "peer"))
+				{
+					auth = "peer";
+					goto validate;
+				}
+				goto sip404;
+			}
+
 			auth = "none";
 			if(peer && !reg)
 				auth = peer->type;
@@ -1335,6 +1591,22 @@ killit:
 				reg = NULL;
                         if(reg && !proxy && !stricmp(auth, "proxy"))
                                 reg = NULL; 
+
+			if(!reg && !peer)
+			{
+				endImage(img);
+				requestAuth(sevent);
+				goto clear;
+			}
+
+			if(reg && !stricmp(reg->type, "peer"))
+			{
+				endImage(img);
+				requestAuth(sevent);
+				goto clear;
+			}
+
+validate:
 			if(reg)
 			{
 				if(!reg->isActive())
@@ -1364,6 +1636,12 @@ sip404:
 				eXosip_call_send_answer(sevent->tid, 404, NULL);
 				eXosip_unlock();				
 				goto clear;
+sip401:
+				endImage(img);
+				eXosip_lock();
+				eXosip_call_send_answer(sevent->tid, 401, NULL);
+				eXosip_unlock();
+				goto clear;
 			}
 
 			s = getIdle();
@@ -1673,8 +1951,8 @@ sip415:
 			
 			eXosip_unlock();
 			session = NULL;
-
-
+			goto clear;
+		
 clear:
 			if(session)
 				add(session);
