@@ -127,6 +127,10 @@ public:
 
 	void startDriver(void);
 	void stopDriver(void);
+
+	void requestAuth(eXosip_event_t *sevent);
+	bool getRegistry(eXosip_event_t *sevent, Registry **reg, ScriptImage *img);
+	void registerRequest(eXosip_event_t *sevent);
 	const char *registerScript(ScriptImage *img, Line *line);
 	const char *assignScript(ScriptImage *img, Line *line);
 
