@@ -144,7 +144,10 @@ timeout:
 	req_command = strtok_r(request, " \t\r\n", &tok);
 	req_path = strtok_r(NULL, " \t\r\n", &tok);
 	req_query = NULL;
-	tok = strchr(req_path, '?');
+	if(req_path)
+		tok = strchr(req_path, '?');
+	else
+		tok = NULL;
 	if(tok)
 	{
 		*tok = 0;
