@@ -195,7 +195,7 @@ void Session::xmlRegistrations(void)
 			snprintf(buffer, sizeof(buffer), 
 				"   <list id=\"registration\">\r\n"
 				"    <item id=\"driver\" value=\"%s\"/>\r\n"
-				"    <item id=\"server\" value=\"%s\"/>\r\n"
+				"    <item id=\"remote\" value=\"%s\"/>\r\n"
 				"    <item id=\"userid\" value=\"%s\"/>\r\n"
 				"    <item id=\"access\" value=\"%s\"/>\r\n"
 				"    <item id=\"status\" value=\"%s\"/>\r\n"
